/*

Copyright (C) 2000, 2001 Christian Kreibich <kreibich@aciri.org>.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/param.h>
#ifdef LINUX
#define __FAVOR_BSD
#endif
#include <gtk/gtk.h>

#include <nd_debug.h>
#include <nd_prefs.h>
#include <nd_macros.h>
#include <interface.h>

#define ND_CONFIG_FILE ".netdude"
#define ND_NO_SAVE_DIR "(none)"


typedef struct nd_preferences
{
  int     trace_win_size;
  char   *tcpdump_path;

  char   *load_dir;
  char   *save_dir;
}
ND_Preferences;


static GtkWidget *prefs_dialog = NULL;
static ND_Preferences prefs;
static ND_Preferences prefs_backup;

static char *prefs_config_file(void);
static int   prefs_read_config_file(ND_Preferences *p);
static int   prefs_write_config_file(ND_Preferences *p);
static int   prefs_init_defaults(void);
static void  prefs_sync_app_to_settings(void);
static void  prefs_sync_settings_to_dialog(void);
static void  prefs_sync_dialog_to_settings(void);

static char *
prefs_config_file(void)
{
  struct stat st;
  static char cfg_file[MAXPATHLEN] = "\0";

  if (cfg_file[0] != 0)
    return cfg_file;

  snprintf(cfg_file, MAXPATHLEN, "%s/%s", getenv("HOME"), ND_CONFIG_FILE);

  if (stat(cfg_file, &st) < 0)
    return NULL;

  return cfg_file;
}


static int
prefs_read_config_file(ND_Preferences *p)
{
  char *cfg_file = NULL;
  char s[MAXPATHLEN];
  FILE *f;

  if (!p)
    return (FALSE);

  if ((cfg_file = prefs_config_file()) == NULL)
    return (FALSE);

  /* Initialize things properly */
  FREE(p->tcpdump_path);
  FREE(p->load_dir);
  FREE(p->save_dir);

  memset(p, 0, sizeof(ND_Preferences));

  if ((f = fopen(cfg_file, "r")) == NULL)
    return (FALSE);

  for ( ; ; )
    {
      if (fscanf(f, "%s", s) == EOF)
	break;

      if (!strcmp(s, "trace_win_size"))
	{
	  fscanf(f, "%i\n", &(p->trace_win_size));
	}
      else if (!strcmp(s, "tcpdump_path"))
	{
	  fscanf(f, "%s\n", s);
	  p->tcpdump_path = strdup(s);
	}
      else if (!strcmp(s, "load_dir"))
	{
	  fscanf(f, "%s\n", s);
	  p->load_dir = strdup(s);
	}
      else if (!strcmp(s, "save_dir"))
	{
	  fscanf(f, "%s\n", s);
	  if (strcmp(s, ND_NO_SAVE_DIR))
	    p->save_dir = strdup(s);
	}
    }

  fclose(f);
  return (TRUE);
}


static int
prefs_write_config_file(ND_Preferences *p)
{
  FILE *f;

  if (!p)
    return (FALSE);

  if ((f = fopen(prefs_config_file(), "w")) == NULL)
    return (FALSE);

  fprintf(f, "trace_win_size \t\t %i\n", p->trace_win_size);
  fprintf(f, "tcpdump_path \t\t %s\n", p->tcpdump_path);
  fprintf(f, "load_dir \t\t %s\n", p->load_dir);
  if (!p->save_dir)
    fprintf(f, "save_dir \t\t %s\n", ND_NO_SAVE_DIR);
  else
    fprintf(f, "save_dir \t\t %s\n", p->save_dir);

  fclose(f);
  return (TRUE);
}


static int
prefs_init_defaults(void)
{
  char s[MAXPATHLEN];

  snprintf(s, MAXPATHLEN, "%s/", getenv("HOME"));
  memset(&prefs, 0, sizeof(ND_Preferences));

  prefs.trace_win_size = 1000;
  prefs.tcpdump_path = strdup(TCPDUMP);
  prefs.load_dir = strdup(s);
  prefs.save_dir = NULL;
  prefs_write_config_file(&prefs);

  return (TRUE);
}


static void  
prefs_sync_app_to_settings(void)
{
}


static void  
prefs_sync_settings_to_dialog(void)
{
  GtkWidget *w;

  D_ASSERT(prefs_dialog);

  w = gtk_object_get_data(GTK_OBJECT(prefs_dialog), "trace_win_size_spin");
  D_ASSERT(w);
  prefs.trace_win_size = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
}


static void
prefs_sync_dialog_to_settings(void)
{
  GtkWidget *w;
  char      tcpdump_path[MAXPATHLEN];

  if (!prefs_dialog)
    return;

  w = gtk_object_get_data(GTK_OBJECT(prefs_dialog), "trace_win_size_spin");
  if (w)
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), prefs.trace_win_size); 

  w = gtk_object_get_data(GTK_OBJECT(prefs_dialog), "pref_tcpdump_loc_entry");
  if (w)
    {
      if (nd_prefs_get_item("tcpdump_path", tcpdump_path))
	{
	  gtk_entry_set_text(GTK_ENTRY(w), tcpdump_path);
	}
    }
}


int
nd_prefs_init(void)
{
  int result = TRUE;
  char *cfg_file = NULL;

  memset(&prefs, 0, sizeof(ND_Preferences));

  if ((cfg_file = prefs_config_file()) != NULL)
    result = prefs_read_config_file(&prefs);
  else
    result = prefs_init_defaults();

  return (result);
}


int
nd_prefs_save(void)
{
  return (prefs_write_config_file(&prefs));
}


void
nd_prefs_save_general(void)
{
  ND_Preferences p;

  /* Load the preferences as currently on disk,
     and overwrite the user-independent settings.
     Then write back to disk.
  */

  memset(&p, 0, sizeof(ND_Preferences));
  prefs_read_config_file(&p);

  FREE(p.load_dir);
  FREE(p.save_dir);
  p.load_dir = prefs.load_dir;
  p.save_dir = prefs.save_dir;
  
  /* Needn't free the strings here */

  prefs_write_config_file(&p);
}


int            
nd_prefs_get_item(const char *key, void *data)
{
  int result = TRUE;

  if (!key || key[0] == '\0' || !data)
    return (FALSE);

  if (!strcmp(key, "trace_win_size"))
    {
      int *i = (int*)data;

      *i = prefs.trace_win_size;
    }
  else if (!strcmp(key, "tcpdump_path"))
    {
      char *s = (char*)data;

      if (prefs.tcpdump_path)
	snprintf(s, MAXPATHLEN, prefs.tcpdump_path);
      else
	result = FALSE;
    }
  else if (!strcmp(key, "load_dir"))
    {
      char *s = (char*)data;
      
      if (prefs.load_dir)
	snprintf(s, MAXPATHLEN, prefs.load_dir);
      else
	result = FALSE;
    }
  else if (!strcmp(key, "save_dir"))
    {
      char *s = (char*)data;

      if (prefs.save_dir)
	snprintf(s, MAXPATHLEN, prefs.save_dir);
      else
	result = FALSE;
    }
  else
    {
      result = FALSE;
    }

  return (result);
}


int            
nd_prefs_set_item(const char *key, void *data)
{
  int result = TRUE;

  if (!key || key[0] == '\0' || !data)
    return (FALSE);

  if (!strcmp(key, "trace_win_size"))
    {
      int *i = (int*)data;
      prefs.trace_win_size = *i;
    }
  else if (!strcmp(key, "tcpdump_path"))
    {
      char *s = (char*)data;
      FREE(prefs.tcpdump_path);
      prefs.tcpdump_path = strdup(s);
    }
  else if (!strcmp(key, "load_dir"))
    {
      char *s = (char*)data;
      FREE(prefs.load_dir);
      prefs.load_dir = strdup(s);
    }
  else if (!strcmp(key, "save_dir"))
    {
      char *s = (char*)data;
      FREE(prefs.save_dir);
      prefs.save_dir = strdup(s);
    }
  else
    {
      result = FALSE;
    }

  prefs_sync_dialog_to_settings();

  return (result);
}

void     
nd_prefs_dialog_show(void)
{
  memcpy(&prefs_backup, &prefs, sizeof(ND_Preferences));

  if (!prefs_dialog)
    prefs_dialog = create_pref_dialog();

  prefs_sync_dialog_to_settings();
  gtk_widget_show(prefs_dialog);
}


void     
nd_prefs_dialog_ok(void)
{
  prefs_sync_settings_to_dialog();
  prefs_sync_app_to_settings();

  if (!prefs_dialog)
    return;

  gtk_widget_hide(prefs_dialog);
}


void     
nd_prefs_dialog_save(void)
{
  prefs_sync_settings_to_dialog();
  prefs_write_config_file(&prefs);
}


void     
nd_prefs_dialog_cancel(void)
{
  memcpy(&prefs, &prefs_backup, sizeof(ND_Preferences));

  if (!prefs_dialog)
    return;

  gtk_widget_hide(prefs_dialog);
}

